@@ -57,9 +57,7 @@ app_path_for_id (const gchar *app_id)
 static gboolean
 handle_add_notification (XdpImplNotification *object,
                          GDBusMethodInvocation *invocation,
-#ifdef HAVE_XDP_1_19_1
                          GUnixFDList *fds,
-#endif
                          const gchar *arg_app_id,
                          const gchar *arg_id,
                          GVariant *arg_notification)
@@ -72,11 +70,7 @@ handle_add_notification (XdpImplNotification *object,
                                                notification_added,
                                                NULL);
 
-#ifdef HAVE_XDP_1_19_1
   xdp_impl_notification_complete_add_notification (object, invocation, NULL);
-#else
-  xdp_impl_notification_complete_add_notification (object, invocation);
-#endif
 
   return TRUE;
 }
