@@ -22,8 +22,10 @@
 #include "shell-dbus.h"
 #include "shellintrospect.h"
 
-struct _Window
+struct _ShellWindow
 {
+  GObject parent;
+
   uint64_t id;
   char *title;
   char *app_id;
@@ -40,9 +42,10 @@ struct _ShellIntrospect
 
   unsigned int version;
 
-  GPtrArray *windows;
 
   int num_listeners;
+  GListStore *windows;
+  gboolean initialized;
 
   gboolean animations_enabled;
   gboolean animations_enabled_valid;
@@ -62,96 +65,265 @@ static guint signals[N_SIGNALS];
 
 static ShellIntrospect *_shell_introspect;
 
-void
-window_free (Window *window)
+
+struct _WindowClass
+{
+  GObjectClass parent;
+};
+
+G_DEFINE_TYPE (ShellWindow, shell_window, G_TYPE_OBJECT)
+
+enum
+{
+  PROP_0,
+  PROP_ID,
+  PROP_TITLE,
+  PROP_APP_ID,
+  N_PROPS
+};
+
+GParamSpec *properties[N_PROPS] = {NULL, };
+
+static void
+shell_window_init (ShellWindow *obj)
 {
-  g_free (window->title);
-  g_free (window->app_id);
-  g_free (window);
 }
 
-Window *
-window_dup (Window *window)
+static void
+shell_window_dispose (GObject *obj)
 {
-  Window *new_window;
+  ShellWindow *self = SHELL_WINDOW (obj);
 
-  new_window = g_new0 (Window, 1);
-  new_window->id = window->id;
-  new_window->title = g_strdup (window->title);
-  new_window->app_id = g_strdup (window->app_id);
+  g_clear_pointer (&self->title, g_free);
+  g_clear_pointer (&self->app_id, g_free);
+}
+
+static void
+shell_window_set_title (ShellWindow *window,
+                        const char  *title)
+{
+  g_clear_pointer (&window->title, g_free);
+
+  if (title == NULL)
+    window->title = g_strdup ("");
+  else
+    window->title = g_markup_escape_text (title, -1);
+
+  g_object_notify (G_OBJECT (window), "title");
+}
+
+static void
+shell_window_get_property (GObject    *object,
+                           guint       property_id,
+                           GValue     *value,
+                           GParamSpec *pspec)
+{
+  ShellWindow *window = SHELL_WINDOW (object);
+
+  switch (property_id)
+    {
+    case PROP_ID:
+      g_value_set_uint64 (value, window->id);
+      break;
+
+    case PROP_TITLE:
+      g_value_set_string (value, window->title);
+      break;
+
+    case PROP_APP_ID:
+      g_value_set_string (value, window->app_id);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+}
+
+static void
+shell_window_set_property (GObject      *object,
+                           guint         property_id,
+                           const GValue *value,
+                           GParamSpec   *pspec)
+{
+  ShellWindow *window = SHELL_WINDOW (object);
+
+  switch (property_id)
+    {
+    case PROP_ID:
+      window->id = g_value_get_uint64 (value);
+      break;
+
+    case PROP_TITLE:
+      shell_window_set_title (window, g_value_get_string (value));
+      break;
+
+    case PROP_APP_ID:
+      window->app_id = g_strdup (g_value_get_string (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+}
+
+
+static void
+shell_window_class_init (ShellWindowClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->dispose = shell_window_dispose;
+  object_class->get_property = shell_window_get_property;
+  object_class->set_property = shell_window_set_property;
+
+  properties[PROP_ID] = g_param_spec_uint64 ("id", NULL, NULL, 0, G_MAXUINT64, 0,
+                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+  properties[PROP_TITLE] = g_param_spec_string ("title", NULL, NULL, "",
+                                                G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+  properties[PROP_APP_ID] = g_param_spec_string ("app_id", NULL, NULL, "",
+                                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
+}
+
+static ShellWindow *
+shell_window_new (uint64_t  id,
+                  char     *title,
+                  char     *app_id)
+{
+  ShellWindow *new_window;
+
+  new_window = g_object_new (SHELL_TYPE_WINDOW, "id", id,
+                             "title", title,
+                             "app_id", app_id, NULL);
 
   return new_window;
 }
 
-static void
-get_windows_cb (GObject *source_object,
-                GAsyncResult *res,
-                gpointer user_data)
+ShellWindow *
+shell_window_dup (ShellWindow *window)
 {
-  ShellIntrospect *shell_introspect = user_data;
+  return shell_window_new (window->id, window->title, window->app_id);
+}
+
+static ShellWindow *
+find_window_from_id (ShellIntrospect *shell_introspect,
+                     uint64_t         id,
+                     unsigned int    *index)
+{
+  GListModel *window_list = G_LIST_MODEL (shell_introspect->windows);
+
+  for (unsigned int i = 0; i < g_list_model_get_n_items (window_list); i++)
+    {
+      ShellWindow *window =
+        SHELL_WINDOW (g_list_model_get_item (window_list, i));
+
+      if (id == shell_window_get_id (window))
+        {
+          if (index)
+            *index = i;
+          return window;
+        }
+    }
+
+  return NULL;
+}
+
+static void
+update_window_list (ShellIntrospect *shell_introspect,
+                    GVariant        *windows_variant)
+{
+  GListModel *window_list = G_LIST_MODEL (shell_introspect->windows);
   g_autoptr(GPtrArray) windows = NULL;
-  g_autoptr(GVariant) windows_variant = NULL;
   g_autoptr(GError) error = NULL;
   GVariantIter iter;
   uint64_t id;
   GVariant *params = NULL;
+  unsigned int i;
+  g_autoptr(GHashTable) old_ids = NULL;
+  gpointer key;
+  GHashTableIter old_ids_iter;
 
-  g_clear_object (&shell_introspect->cancellable);
-
-  if (!org_gnome_shell_introspect_call_get_windows_finish (shell_introspect->proxy,
-                                                           &windows_variant,
-                                                           res,
-                                                           &error))
+  old_ids = g_hash_table_new_full (g_int64_hash, g_int64_equal, g_free, NULL);
+  for (i = 0; i < g_list_model_get_n_items (window_list); i++)
     {
-      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-        g_warning ("Failed to get window list: %s", error->message);
-      return;
+      ShellWindow *window =
+        SHELL_WINDOW (g_list_model_get_item (window_list, i));
+      uint64_t id = shell_window_get_id (window);
+
+      g_hash_table_add (old_ids, g_memdup2 (&id, sizeof (id)));
     }
 
   g_variant_iter_init (&iter, windows_variant);
 
   windows = g_ptr_array_new_full (g_variant_iter_n_children (&iter),
-                                  (GDestroyNotify) window_free);
+                                  (GDestroyNotify) g_object_unref);
 
   while (g_variant_iter_loop (&iter, "{t@a{sv}}", &id, &params))
     {
-      char *app_id = NULL;
-      char *title = NULL;
+      g_autofree char *app_id = NULL;
+      g_autofree char *title = NULL;
       unsigned int time_since_user_time = UINT_MAX;
-      Window *window;
+      ShellWindow *window;
 
       g_variant_lookup (params, "app-id", "s", &app_id);
       g_variant_lookup (params, "title", "s", &title);
       g_variant_lookup (params, "time-since-user-time", "u", &time_since_user_time);
 
-      window = g_new0 (Window, 1);
-      *window = (Window) {
-        .id = id,
-        .title = title,
-        .app_id = app_id
-      };
-      g_ptr_array_add (windows, window);
+      window = find_window_from_id (shell_introspect, id, NULL);
+      if (window)
+        {
+          g_hash_table_remove (old_ids, &id);
+          shell_window_set_title (window, title);
+        }
+      else
+        {
+          window = shell_window_new (id, title, app_id);
+          g_ptr_array_add (windows, window);
+        }
 
       g_clear_pointer (&params, g_variant_unref);
     }
 
-  shell_introspect->windows = g_steal_pointer (&windows);
+  g_hash_table_iter_init (&old_ids_iter, old_ids);
+  while (g_hash_table_iter_next (&old_ids_iter, &key, NULL))
+    {
+      uint64_t old_id = *(uint64_t *) key;
+      unsigned int index;
+
+      if (find_window_from_id (shell_introspect, old_id, &index))
+        g_list_store_remove (shell_introspect->windows, index);
+    }
+
+  g_list_store_splice (shell_introspect->windows, 0, 0, windows->pdata, windows->len);
+  shell_introspect->initialized = TRUE;
+
   g_signal_emit (shell_introspect, signals[WINDOWS_CHANGED], 0);
 }
 
 static void
 sync_state (ShellIntrospect *shell_introspect)
 {
-  g_clear_pointer (&shell_introspect->windows, g_ptr_array_unref);
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GVariant) windows_variant = NULL;
 
   g_cancellable_cancel (shell_introspect->cancellable);
   g_clear_object (&shell_introspect->cancellable);
   shell_introspect->cancellable = g_cancellable_new ();
 
-  org_gnome_shell_introspect_call_get_windows (shell_introspect->proxy,
-                                               shell_introspect->cancellable,
-                                               get_windows_cb,
-                                               shell_introspect);
+  if (!org_gnome_shell_introspect_call_get_windows_sync (shell_introspect->proxy,
+                                                         &windows_variant,
+                                                         shell_introspect->cancellable,
+                                                         &error))
+    {
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+        g_warning ("Failed to get window list: %s", error->message);
+      return;
+    }
+
+  g_clear_object (&shell_introspect->cancellable);
+
+  update_window_list (shell_introspect, windows_variant);
 }
 
 static void
@@ -277,6 +449,7 @@ shell_introspect_class_init (ShellIntrospectClass *klass)
 static void
 shell_introspect_init (ShellIntrospect *shell_introspect)
 {
+  shell_introspect->windows = g_list_store_new (SHELL_TYPE_WINDOW);
 }
 
 ShellIntrospect *
@@ -299,10 +472,10 @@ shell_introspect_get (void)
   return shell_introspect;
 }
 
-GPtrArray *
+GListModel *
 shell_introspect_get_windows (ShellIntrospect *shell_introspect)
 {
-  return shell_introspect->windows;
+  return G_LIST_MODEL (shell_introspect->windows);
 }
 
 void
@@ -321,23 +494,23 @@ shell_introspect_unref_listeners (ShellIntrospect *shell_introspect)
 
   shell_introspect->num_listeners--;
   if (shell_introspect->num_listeners == 0)
-    g_clear_pointer (&shell_introspect->windows, g_ptr_array_unref);
+    g_list_store_remove_all (shell_introspect->windows);
 }
 
 const char *
-window_get_title (Window *window)
+shell_window_get_title (ShellWindow *window)
 {
   return window->title;
 }
 
 const char *
-window_get_app_id (Window *window)
+shell_window_get_app_id (ShellWindow *window)
 {
   return window->app_id;
 }
 
 const uint64_t
-window_get_id (Window *window)
+shell_window_get_id (ShellWindow *window)
 {
   return window->id;
 }
@@ -360,6 +533,6 @@ shell_introspect_wait_for_windows (ShellIntrospect *shell_introspect)
 
   sync_state (shell_introspect);
 
-  while (!shell_introspect->windows)
+  while (!shell_introspect->initialized)
     g_main_context_iteration (NULL, TRUE);
 }
